#!/bin/bash
set -ex

source charms.reactive.sh

ARCHIVE_DIR=$CHARM_DIR/files/archives
SYMPHONY_INSTALL_PATH=/opt/ibm/spectrumsymphony
CLUSTER_ADMIN=egoadmin
hostname_master=`hostname`
ip_master=`unit-get private-address`
ARCHITECTURE=`uname -m`
hook_compare=relation-departed
master_reconfig_flag="Nil"

@when_not 'ibm-spectrum-symphony-master.nfsclient.installed'
function install_nfsclient(){
	juju-log "IBM Spectrum Symphony: Begin Install."
	# To check for unsupported platforms
	if [ "$ARCHITECTURE" != "x86_64" -a "$ARCHITECTURE" != "ppc64le" ]; then
		juju-log "IBM Spectrum Symphony : Unsupported platform. IBM Spectrum Symphony installed with this Charm supports only the x86_64 platform and POWER LE (ppc64le) platforms."
		return 1
	fi
	# Install nfs-common using apt
	apt-get install nfs-common -y
	set_state 'ibm-spectrum-symphony-master.nfsclient.installed'
	remove_state 'ibm-spectrum-symphony-master.mastercandt.started'
	remove_state 'ibm-spectrum-symphony-master.masterconf.ready'
	remove_state 'ibm-spectrum-symphony-master.ready'
	remove_state 'ibm-spectrum-symphony-master.nfsclient.ready'
	juju-log "IBM Spectrum Symphony: Pre-reqs installed successfully"
}


function set_entitlement(){

	su - $CLUSTER_ADMIN -c ". $SYMPHONY_INSTALL_PATH/profile.platform;hostname_master=`hostname`;egoconfig join $hostname_master -f;egoconfig setentitlement $SYMPHONY_INSTALL_PATH/*.dat"
	if [ "$?" == "0" ]
	then
		juju-log "IBM Spectrum Symphony: configuration steps completed"
	else
		juju-log "IBM Spectrum Symphony: Error during configuration steps"
		return 1 
	fi
}



# Function to add hostname details to /etc/hosts file
function add_hostname(){
	# Block of code to get the IP and hostname of Master/Master Candidate Host machine
	juju-log "IBM Spectrum Symphony : Master/Master Candidate host Hostname is : $hostname_master"
	juju-log "IBM Spectrum Symphony : Master host IP is : $ip_master"
	# Adding hostname entry in hosts file
	cd /etc
	if grep -w "$ip_master $hostname_master" hosts
	then
		juju-log "IBM Spectrum Symphony : Host file already updated"                                         
	else
		juju-log "IBM Spectrum Symphony : Updating Host file"                                           
		echo "$ip_master $hostname_master" >> /etc/hosts
	fi
}


# Function to find symphony version and distribution type
function find_version_distr()
{
	# To find the symphony version
	if ! [ -d "$SYMPHONY_INSTALL_PATH/kernel/conf" ]; then
		juju-log "IBM Spectrum Symphony : Symphony Install Directory does not exist, Symphony Folders are not properly mounted, hence exiting."
		return 0
	fi

	ps_version=`grep "EGO_VERSION" $SYMPHONY_INSTALL_PATH/kernel/conf/ego.conf | awk -F'=' '{print $2}'`
	# To determine the distribution type of host node
	ARCHITECTURE=`uname -m`
	cd $SYMPHONY_INSTALL_PATH/$ps_version
	if [ "$ARCHITECTURE" = "x86_64" ]; then
		distribution_OS_type=`ls -lrt | grep x86_64 | awk -F' ' '{print $9}'`
	elif [ "$ARCHITECTURE" = "ppc64le" ]; then
		distribution_OS_type=`ls -lrt | grep ppc64le | awk -F' ' '{print $9}'`
	fi 
}


# Block of code for adding the master host to the Symphony storage server and share the Symphony installation binaries using NFS.
function nfs_sharesymphony(){
	# Using NFS , share the Symphony file location installed on storage server and mount it on master host
	ip_storage=$1
	if  [ -d $SYMPHONY_INSTALL_PATH ] ; then
		juju-log "IBM Spectrum Symphony :Symphony Directory already exists" 
	else
		mkdir -p $SYMPHONY_INSTALL_PATH
	fi

	if grep -q "$ip_storage:$SYMPHONY_INSTALL_PATH" /etc/fstab
	then
		juju-log "IBM Spectrum Symphony : Entries already exist in /etc/fstab for mounting"
	else
		echo "$ip_storage:$SYMPHONY_INSTALL_PATH $SYMPHONY_INSTALL_PATH nfs rw 0 0" >> /etc/fstab 
	fi

	mount -a
	if [ "$?" != "0" ]; then
		juju-log "IBM Spectrum Symphony : Error while mounting the Symphony Directory, check the /etc/fstab or check whether the Storage Server is ready"
		return 1
	else
		juju-log "IBM Spectrum Symphony : Successfully mounted the Symphony Directory"
	fi
}

# Function to create symphony admin user and sudoers file
function createuser_conf(){
	# checking whether egoadmin already exists or not
	cd /etc
	if grep -q "egoadmin" passwd 
	then
		juju-log "IBM Spectrum Symphony : User egoadmin already exists"
	else
		useradd -m -d /home/$CLUSTER_ADMIN $CLUSTER_ADMIN
		if [ $? == 0 ]; then
			juju-log "IBM Spectrum Symphony : Symphony administrator egoadmin user created successfully"
		else
			juju-log "IBM Spectrum Symphony : Error while creating the Symphony administrator user "
			return 0
		fi
	fi
       
	find_version_distr
	cd /etc
	if [[ "`grep "EGO_STARTUP_USERS" ego.sudoers`" ]] && [[ "`grep "EGO_STARTUP_PATH" ego.sudoers`" ]]; then
		juju-log "IBM Spectrum Symphony : ego.sudoers file already updated with values"
	else               
		touch ego.sudoers
		echo "EGO_STARTUP_PATH=$SYMPHONY_INSTALL_PATH/$ps_version/$distribution_OS_type/etc" >> ego.sudoers
		sed -i '1i EGO_STARTUP_USERS="egoadmin"' ego.sudoers
		chmod 600 ego.sudoers
		juju-log "IBM Spectrum Symphony : ego.sudoers file updated."
	fi
}



function create_master_test(){
	# Code to find out whether this is the first instance getting connected to Storage Server
	juju-log "IBM Spectrum Symphony : The hostname of Symphony storage is : $hostname_storage"
	master_exists=`grep EGO_MASTER_LIST $SYMPHONY_INSTALL_PATH/kernel/conf/ego.conf | awk -F"EGO_MASTER_LIST="  '{print $2}' | sed 's/\"//g'`
	if [ -n  $master_exists ]; then		
		if grep -q "$hostname_master" $SYMPHONY_INSTALL_PATH/kernel/conf/ego.conf
		then
			juju-log "IBM Spectrum Symphony : Hostname already exists in ego.conf file"
		else
			set_entitlement
		fi
		start_symphony
		set_state 'ibm-spectrum-symphony-master.ready'
		status-set active "Symphony Master is ready"
	else
		juju-log "IBM Spectrum Symphony : This is a master candidate"
		set_state 'ibm-spectrum-symphony-master.masterconf.ready'
	fi
}


# When Storage server is not available
@when_not 'storage.joined' 'ibm-spectrum-symphony-master.nfsclient.ready'
function notify_user(){
	juju-log "IBM Spectrum Symphony : Please add a relation between Symphony Storage and Symphony Master charm to have a working Symphony Master"
	status-set waiting "Waiting for relation with Symphony Storage/Details not sent from Storage Server"
}


# Function to mount the Symphony Installation files when relation with Storage Server is available
@when 'storage.available' 
@when_not 'ibm-spectrum-symphony-master.nfsclient.ready'
function configure_master(){
	juju-log "IBM Spectrum Symphony: Mounting of shared Symphony Installation files begins ..."
	ip=$(relation_call --state=storage.available private_address) || true 
	juju-log "IBM Spectrum Symphony : IP of Symphony Storage Server is $ip"
	hostname_storage=$(relation_call --state=storage.available get_hostname) || true
	juju-log "IBM Spectrum Symphony : Hostname Symphony Storage Server is $hostname_storage"
	add_hostname
	nfs_sharesymphony $ip
	juju-log "IBM Spectrum Symphony : NFS Client is ready now and Symphony Files have been shared."
	status-set maintenance "NFS client is ready and Symphony Files shared"
	set_state 'ibm-spectrum-symphony-master.nfsclient.ready'
	createuser_conf
	create_master_test
}


# Function to start Symphony Daemons, this will be called when Symphony configuration is done and daemons needs to be started
function start_symphony(){
	juju-log " IBM Spectrum Symphony : Starting the Symphony Daemons begins"
	# Check if symphony files are available
	if ! [ -d "$SYMPHONY_INSTALL_PATH/kernel/conf" ]; then
		juju-log "IBM Spectrum Symphony : Symphony Install Directory does not exist"
		return 0
	fi
	juju-log "IBM Spectrum Symphony : Setting the Symphony profile path"
	set +e
	. $SYMPHONY_INSTALL_PATH/profile.platform
	egosh ego start
	if [ "$?" == "0" ]; then
		juju-log "Master Started"
		status-set active "Master Started"
	else
		juju-log "Error Starting Master"
		return 1
	fi
	co=0
	juju-log "Logging to Master......"
	egosh user logon -u Admin -x Admin
	status=`echo $?`
	if [ $status = "0" ]; then
		juju-log "Logged on"
	else
		juju-log "Failed to Logon.Trying again..."
	fi
	while [ ! $status = 0 ] && [ $co -lt 5 ]
	do
		sleep 30
		egosh user logon -u Admin -x Admin
		status=`echo $?`
		if [ $status = "0" ]; then
			juju-log "Logged on"
		fi
		co=$co+1
	done
	egosh resource list -l
	co=0
	set -e

}

# Function to stop Symphony Daemons
function stop_symphony(){
	set +e
	. $SYMPHONY_INSTALL_PATH/profile.platform
	egosh service stop all && egosh ego shutdown -f
	 set -e
	if [ "$?" != "0" ]; then
		juju-log "IBM Spectrum Symphony : Error while stopping Symphony Cluster"
	else
		juju-log "IBM Spectrum Symphony : Symphony Cluster stopped successfully"
	fi
}

# Function for reconfiguring Symphony Master when a Symphony node is added to the Symphony Cluster
function symphonyfilechanged_server(){
	juju-log "IBM Spectrum Symphony : Reconfiguring Symphony Master once server details are added in ego.cluster.cluster1 file"
	if ! [ -d "$SYMPHONY_INSTALL_PATH/kernal/conf" ]; then
		juju-log "IBM Spectrum Symphony : Symphony Install Directory does not exist, Symphony Folders are not properly mounted, hence exiting."
		return 0
	fi
	. $SYMPHONY_INSTALL_PATH/profile.platform
	egosh user logon -u Admin -x Admin
	status=`echo $?`
	if [ $status = "0" ]; then
		juju-log "Logged on"
	else
		juju-log "Failed to Logon.Trying again..."
	fi
	while [ ! $status = 0 ] && [ $co -lt 5 ]
	do
		sleep 30
		egosh user logon -u Admin -x Admin
		status=`echo $?`
		if [ $status = "0" ]; then
			juju-log "Logged on"
		fi
		co=$co+1
	done
	curr_mast_hostname=`egosh resource list -m | grep "EGO current master host name" | awk -F"EGO current master host name : "  '{print $2}'`
	juju-log "IBM Spectrum Symphony : The current master of the Symphony Cluster is : $curr_mast_hostname"
	if [ "$curr_mast_hostname" == "$hostname_master" ]; then
		egosh service stop all && egosh ego restart -f
	fi
}


# Function for sending the hostname details to Symphony Server when it wants to join the Symphony Cluster
@when 'master.joined'
function send_hostname_toServer(){
	juju-log "IBM Spectrum Symphony : Hostname of Master server is : $hostname_master"
	relation_call --state=master.joined host_name $hostname_master || true
}


# Function for adding Symphony Server hosts to the Symphony Cluster
@when 'master.available' 
@when_any 'ibm-spectrum-symphony-master.ready' 'ibm-spectrum-symphony-master.mastercandt.started'
@when_not 'cluster.departed'
function add_hostname_server(){
	#Add the hostname details in /etc/hosts file
	juju-log "IBM Spectrum Symphony : Hostname of Master server is : $hostname_master"
	juju-log "IBM Spectrum Symphony : Setting the Symphony profile path"
	set +e
	. $SYMPHONY_INSTALL_PATH/profile.platform
	egosh user logon -u Admin -x Admin
	curr_mast_hostname=`egosh resource list -m | grep "EGO current master host name" | awk -F"EGO current master host name : "  '{print $2}'`
	set -e
	juju-log "IBM Spectrum Symphony : Current master is : $curr_mast_hostname"
	count=0
	count1=0
	hostname_server=$(relation_call --state=master.available server_hostname) || true  
	ip_server=$(relation_call --state=master.available private_address) || true  
	juju-log "IBM Spectrum Symphony : IP and hostname of the server and will be adding it to the Hosts file of Master......: $hostname_server $ip_server" 
	for server_ip in $ip_server
		do
		for server_hostname in $hostname_server
			do
			 if [ $count == $count1 ]; then
				cd /etc
				current_hook=`echo $JUJU_HOOK_NAME`
			if [ "$curr_mast_hostname" == "$hostname_master" ] && [[ $current_hook != *"$hook_compare" ]]; then
				if grep -q "$server_hostname" $SYMPHONY_INSTALL_PATH/kernal/conf/ego.cluster.cluster1 
				then
					juju-log "IBM Spectrum Symphony : Symphony Cluster.cluster1 file already updated with host information of the Symphony Server" 
					if grep -w "$server_ip $server_hostname" hosts
					then
						juju-log "IBM Spectrum Symphony : Host file already updated with host information of the Symphony Server"
							master_reconfig_flag="MasterReConfigDone"
					else
						juju-log "IBM Spectrum Symphony : Updating Host file"                                           
						echo "$server_ip $server_hostname" >> /etc/hosts
							juju-log "IBM Spectrum Symphony : This node is the current master as of now and it will be reconfigured"
						symphonyfilechanged_server
						master_reconfig_flag="MasterReConfigReady"
					fi
				else
					juju-log "IBM Spectrum Symphony : Updating the Symphony Cluster.cluster1 file"                                           
					sed -i '/End     Host/c\'$server_hostname' !        !            -    -    -   (linux)\nEnd     Host' $SYMPHONY_INSTALL_PATH/kernel/conf/ego.cluster.cluster1
					if grep -w "$server_ip $server_hostname" hosts
					then
						juju-log "IBM Spectrum Symphony : Host file already updated with host information of the Symphony Server"                          
					else
						juju-log "IBM Spectrum Symphony : Updating Host file"                                           
						echo "$server_ip $server_hostname" >> /etc/hosts
					fi
						juju-log "IBM Spectrum Symphony : This host is the current master as of now and it will be reconfigured"
					symphonyfilechanged_server
					master_reconfig_flag="MasterReConfigReady"
				fi
					elif [ "$curr_mast_hostname" != "$hostname_master" ] && [ "$current_hook" != "master-relation-departed" ]; then
					if grep -w "$server_ip $server_hostname" hosts
					then
						juju-log "IBM Spectrum Symphony : Host file already updated with host information of the Symphony Server"                           
					else
						juju-log "IBM Spectrum Symphony : Updating Host file"                                           
						echo "$server_ip $server_hostname" >> /etc/hosts
					fi
					master_reconfig_flag="NotMaster"
				fi
				relation_call --state=master.available notify_server $master_reconfig_flag || true 
			fi
			count1=$((count1+1))
		done
		count=$((count+1))
		count1=0
	done 
}


@when 'cluster.joined' 
function cluster_send_currmasterconf(){
	juju-log "IBM Spectrum Symphony : Hostname of current node is : $hostname_master"
	relation_call --state=cluster.joined set_unithost $hostname_master || true 
	relation_call --state=cluster.joined dismiss_joined || true
}


@when 'cluster.available' 
@when_any 'ibm-spectrum-symphony-master.ready' 'ibm-spectrum-symphony-master.masterconf.ready'
function cluster_get_masterconf(){
	juju-log "IBM Spectrum Symphony : Hostname of current node is : $hostname_master"
	set +e
	start_symphony
	curr_mast_hostname=`egosh resource list -m | grep "EGO current master host name" | awk -F"EGO current master host name : "  '{print $2}'`
	set -e
	juju-log "IBM Spectrum Symphony : Current master is : $curr_mast_hostname"
	hook_compare=relation-departed
	ip_node=$(relation_call --state=cluster.available get_unitip) || true
	host_node=$(relation_call --state=cluster.available get_unithost) || true
	#Add the hostname details in /etc/hosts file
	count=0
	count1=0
	for peer_ip in $ip_node
	do
		for peer_hostname in $host_node
		do
			if [ $count == $count1 ]; then
				juju-log "IBM Spectrum Symphony : Value of peer ip is : $peer_ip"
				juju-log "IBM Spectrum Symphony : Value of peer hostname is : $peer_hostname"
				current_hook=`echo $JUJU_HOOK_NAME`
				cd /etc
				master_exists=`grep EGO_MASTER_LIST $SYMPHONY_INSTALL_PATH/kernel/conf/ego.conf | awk -F"EGO_MASTER_LIST="  '{print $2}' | sed 's/\"//g'`
				if [ -n $master_exists ]; then
					juju-log "IBM Spectrum Symphony : Master node is removed, so any new added unit will be master"
						if [ "$curr_mast_hostname" == "$hostname_master" ] && [[ $current_hook != *"$hook_compare" ]]; then
							if grep -q "$peer_hostname" $SYMPHONY_INSTALL_PATH/kernel/conf/ego.conf
							then
								juju-log "IBM Spectrum Symphony : Hostname already exists in ego.conf file"
							else
								su - $CLUSTER_ADMIN -c ". $SYMPHONY_INSTALL_PATH/profile.platform;hostname_master=$peer_hostname;egoconfig join $hostname_master -f;egoconfig setentitlement $SYMPHONY_INSTALL_PATH/*.dat"
							fi
							if grep -w "$peer_ip $peer_hostname" hosts
							then
								juju-log "IBM Spectrum Symphony : Host file already updated with host information of the Symphony Master/Master-Candidates"
							else                                         
								echo "$peer_ip $peer_hostname" >> /etc/hosts
								juju-log "IBM Spectrum Symphony : Updating Host file"  
								symphonyfilechanged
							fi
							elif [ "$curr_mast_hostname" != "$hostname_master" ] && [ "$current_hook" != "master-relation-departed" ]; then
								#symphony daemons are not started, start then
							if grep -w "$peer_ip $peer_hostname" hosts
							then
								juju-log "IBM Spectrum Symphony : Host file already updated with host information of the Symphony Master/Master-Candidates"
							else                                        
								echo "$peer_ip $peer_hostname" >> /etc/hosts
								juju-log "IBM Spectrum Symphony : Updating Host file" 
								start_symphony
								set_state 'ibm-spectrum-symphony-master.mastercandt.started'
								status-set active "Symphony Master Candidate is ready"  
							fi
						fi
						elif [ "$master_exists" == "NoMaster " ]; then
							juju-log "IBM Spectrum Symphony : Wait until master is ready:"
						fi
					fi
				count1=$((count1+1))
			done
			count=$((count+1))
			count1=0
		done 
}

# Reconfigure Symphony Cluster, when any master-candidate joins
@when_file_changed  '/etc/hosts'
function symphonyfilechanged(){
	juju-log "IBM Spectrum Symphony : Reconfigure the master node once master candidate is started."
	if ! [ -d "$SYMPHONY_INSTALL_PATH/kernel/conf" ]; then
		juju-log "IBM Spectrum Symphony : Symphony Install Directory does not exist, Symphony Folders are not properly mounted, hence exiting."
		return 0
	fi
	find_version_distr
	juju-log "IBM Spectrum Symphony : Setting the Symphony profile path"
	set +e
	start_symphony
	curr_mast_hostname=`egosh resource list -m | grep "EGO current master host name" | awk -F"EGO current master host name : "  '{print $2}'`
	set -e
	if [ "$curr_mast_hostname" == "$hostname_master" ]; then
		# reconfigure Symphony Master
		juju-log "IBM Spectrum Symphony : Reconfiguring master"
		set +e
		egosh service stop all && egosh ego restart -f
		set -e
	fi
}

@when 'cluster.departed'
function cluster_depart(){
	juju-log "IBM Spectrum Symphony : In Cluster departed function"
	relation_call --state=cluster.departed dismiss_departed || true
}
# When relation between Symphony Storage and Symphony Master is removed or symphony master unit is removed
@when 'storage.departing' 'ibm-spectrum-symphony-master.nfsclient.ready'
@when_not 'ibm-spectrum-symphony-master.storage.departed'
function nfsstorage_depart(){
	#remove_serverhost
	master_exists=`grep $hostname_master $SYMPHONY_INSTALL_PATH/kernel/conf/ego.conf | awk -F'(' '{print $2}' | awk -F')' '{print $1}'`
	juju-log "IBM Spectrum Symphony : Hostname of the master host : $master_exists"
	if ! [ -z  $master_exists ]; then
		juju-log "IBM Spectrum Symphony : Symphony master is getting removed."
		set -e
		stop_symphony
		set +e
		# remove master hostname entry from cluster.cluster1 file
		sed -i  '/'$hostname_master'/d' $SYMPHONY_INSTALL_PATH/kernel/conf/ego.cluster.cluster1
		sed -ri 's/ '$hostname_master'|'$hostname_master' |'$hostname_master'//g' $SYMPHONY_INSTALL_PATH/kernel/conf/ego.conf
		juju-log "IBM Spectrum Symphony : Symphony Master is removed from the cluster, the cluster will be running, but please add a new unit, so that it becomes the new master"
		status-set maintenance "Symphony Master removed, Add new master"
		remove_state 'ibm-spectrum-symphony-master.ready'
		sleep 60
	elif [ "$master_exists" != "$hostname_master" ]; then
		#Stop the Symphony Daemons first
		stop_symphony
		#Remove entry from Symphony.cluster file
		sed -i  '/'$hostname_master'/d' $SYMPHONY_INSTALL_PATH/kernel/conf/ego.cluster.cluster1
		sed -ri 's/ '$hostname_master'|'$hostname_master' |'$hostname_master'//g' $SYMPHONY_INSTALL_PATH/kernel/conf/ego.conf
		remove_state 'ibm-spectrum-symphony-master.mastercandt.started'
		remove_state 'ibm-spectrum-symphony-master.masterconf.ready'
	fi
	# Unmount the Symphony Installation files
	mount_flag=`stat -f -L -c %T $lsf_top` || true
	if  [ "$mount_flag" == "ext2/ext3" ]; then
		juju-log "IBM Spectrum Symphony : shared directory is not mounted yet, cannot proceed with unmounting"
		return 0
	fi
	if ! [ -d $SYMPHONY_INSTALL_PATH/kernel/conf ]; then
		juju-log "IBM Spectrum Symphony : No Symphony folders , nothing to unmount"
	else
	cd /home
	set +e
	umount -f -l $SYMPHONY_INSTALL_PATH 
	set -e
	juju-log " IBM Spectrum Symphony : Unmounted the Symphony installation folders, now Symphony folders will not be accessible"
	sed -i  "\|$SYMPHONY_INSTALL_PATH|d" /etc/fstab     
	remove_state 'ibm-spectrum-symphony-master.nfsclient.ready'
	set_state 'ibm-spectrum-symphony-master.storage.departed'
	fi
}

reactive_handler_main


